import Bladex
import Reference
import Sparks
import Damage
import B3DLib
import ObjStore
import BCopy
import whrandom
import netgame


class SkeletonArmour:
	def __init__(self, me, owner):
		self.ObjId=ObjStore.GetNewId() # Para identificarlo al grabar/guardar
		ObjStore.ObjectsStore[self.ObjId]=self

		self.Name= self.CurrentName= me.Name
		self.OwnerName= owner.Name
		self.AuxFuncsData= None
		self.Age=0
		me.Weapon= 1
		me.CastShadows=1
		me.HitShieldFunc= self.TakeImpact
		inv= owner.GetInventory()
		inv.AddMagicShield(self.Name)############
		
		# armour must be classified as OBJ_SHIELD in DefFuncs and must NOT be selectable #
		kind = Bladex.GetEntity(self.Name).Kind
		if Reference.DefaultObjectData.has_key(kind):
			Reference.EntitiesObjectData[self.Name]= BCopy.deepcopy(Reference.DefaultObjectData[kind])
			self.MaxDefence= self.Defence= Reference.EntitiesObjectData[self.Name][2]
			self.IntDefence= self.MaxDefence/3.0
		else:
			print "Error -- Cannot find default object data for "+kind
			self.MaxDefence= self.Defence= 0
			self.IntDefence= 0
			
		

	def persistent_id(self):
		return self.ObjId

	def __getstate__(self):
		return (1,
				self.ObjId,
				self.Name,
				self.OwnerName,
				self.CurrentName,
				self.MaxDefence,
				self.Defence,
				self.IntDefence,
				self.AuxFuncsData,
				self.Age,
				)

	def __setstate__(self,parm):
		if parm[0]==1:
			self.ObjId=parm[1]
			ObjStore.ObjectsStore[self.ObjId]=self
			self.Name=parm[2]
			self.OwnerName=parm[3]
			self.CurrentName=parm[4]
			self.MaxDefence=parm[5]
			self.Defence=parm[6]
			self.IntDefence=parm[7]
			self.AuxFuncsData=parm[8]
			self.Age=parm[9]
		else:
			print "Version mismatch in BreakSector"
			self.ObjId=ObjStore.GetNewId() # Para identificarlo al grabar/guardar
			ObjStore.ObjectsStore[self.ObjId]=self
			self.Name=""
			self.OwnerName=""
			self.CurrentName=""
			self.MaxDefence=0
			self.Defence=0
			self.IntDefence=0
			self.AuxFuncsData= None
			self.Age=0
		
		if self.Defence>0:
			owner= Bladex.GetEntity(self.OwnerName)
			if owner:
				inv= owner.GetInventory()
				inv.AddMagicShield(self.Name)########
				


	def TakeImpact (self,hit_entity,hitting_entity,xhit_point,yhit_point,zhit_point,ximpulse,yimpulse,zimpulse,DamageType):
		hitting_ent=Bladex.GetEntity(hitting_entity)
		
		hitting_ent.MessageEvent(Reference.MESSAGE_STOP_WEAPON,0,0)
		hitting_ent.MessageEvent(Reference.MESSAGE_STOP_TRAIL,0,0)
		
		hit_ent=Bladex.GetEntity(hit_entity)

		datos_esc=Reference.DefaultObjectData[hit_ent.Kind]
		if datos_esc[0]==Reference.OBJ_SHIELD:
			Reference.DefaultObjectData[hit_ent.Kind][3].Play(xhit_point, yhit_point, zhit_point, 0)
		else:
			if datos_esc[0]==Reference.OBJ_WEAPON:
				if Reference.GiveWeaponFlag(hit_entity)<>Reference.W_FLAG_1H:
					Reference.DefaultObjectData[hit_ent.Kind][5][6].Play(xhit_point, yhit_point, zhit_point, 0)	

		VictimName=self.OwnerName
		AttackerName=hitting_ent.Parent
		pj=Bladex.GetEntity(VictimName)

		
		pj.DamageFunc(VictimName, AttackerName, hitting_entity, DamageType, 0, -1, xhit_point, yhit_point, zhit_point, 1)
		
		
		sparkdir=B3DLib.Normalize((-ximpulse, -yimpulse, -zimpulse))
		chispa=Bladex.CreateSpark("Chispa", xhit_point, yhit_point, zhit_point, sparkdir[0], sparkdir[1], sparkdir[2],0.5,3000,2000,0,100, 160, 120, 40, 30, 24, 0,800,10.0/60.0,1.0/60.0,1)

		if netgame.GetNetState() == 1:
			netgame.CallEventSound(hit_entity,4)

		# Make arrows stick temporarily
		if hitting_ent.Arrow and (not hitting_ent.Parent):
			hitting_ent.Stop()
			# Check type of damage
			impact= hitting_ent.GraspPos ("Impact")
			centre= hitting_ent.Position
			impact2centre= centre[0]-impact[0], centre[1]-impact[1], centre[2]-impact[2]
			abs_pos= xhit_point+impact2centre[0], yhit_point+impact2centre[1], zhit_point+impact2centre[2]
			rel_pos= hit_ent.Abs2RelPoint(abs_pos[0], abs_pos[1], abs_pos[2])
			# To bring it closer to centre, multiply vector by < 1.0
			hitting_ent.Position= rel_pos[0]*0.8, rel_pos[1], rel_pos[2]*0.8
			hit_ent.Link(hitting_ent)
			sticktime= (1.0)/hitting_ent.Mass
			Bladex.AddScheduledFunc (Bladex.GetTime()+sticktime, Damage.StuckWeaponFall, (hitting_ent.Name, hit_ent.Name), hitting_ent.Name+"_StuckWeaponFall")

		


	

	def AbsorbFunc (self, AttackerName, WeaponName, DamagePoints):
		owner=Bladex.GetEntity(self.OwnerName)
		if owner and owner.Life > 0:
			me= Bladex.GetEntity(self.Name)
			self.Defence= max (self.Defence-DamagePoints, 0)
			Reference.EntitiesObjectData[self.Name][2]= self.Defence
			Reference.EntitiesObjectData[self.Name][7]= self.Defence
			#print "Defence remaining "+`self.Defence`

			# play an impact sound
			randsound= whrandom.randint(1,3)
			if randsound==1:	
				SoundName= "..\\..\\Sounds\\golpe-arma-escudo.wav"
			elif randsound==2:
				SoundName= "..\\..\\Sounds\\GOLPE-ARMADUR-1.wav"
			else:
				SoundName= "..\\..\\Sounds\\golpe-maza-armfort.wav"

			self.AGE= self.AGE+1
			impsound=Bladex.CreateEntity (me.Name+"ImpactSound_"+`self.AGE`, "Entity Sound", 0, 0, 0)
			impsound.SetSound (SoundName)
			impsound.MinDistance=5000
			impsound.MaxDistance=60000
			me.Link(impsound)
			impsound.PlaySound(0)
			
			
			
	
	
	
			

			
			
			
			
			
			
			
			
			

			